@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 
 import cinderlib
@@ -26,6 +27,8 @@ class BasePersistenceTest(helper.TestHelper):
 
     def setUp(self):
         super(BasePersistenceTest, self).setUp()
+        self.backend_name = 'fake_backend'
+        self.backend = utils.FakeBackend(volume_backend_name=self.backend_name)
 
     def assertListEqualObj(self, expected, actual):
         exp = [self._convert_to_dict(e) for e in expected]
@@ -386,7 +389,7 @@ class BasePersistenceTest(helper.TestHelper):
     @mock.patch('cinderlib.persistence.base.DB.volume_type_get')
     def test__volume_type_get_by_name(self, get_mock):
         # Only test when using our fake DB class.  We cannot use
-        # unittest2.skipUnless because persistence is configure in setUpClass,
+        # unittest.skipUnless because persistence is configure in setUpClass,
         # which is called after the decorator.
         if not isinstance(cinderlib.objects.Backend.persistence.db,
                           persistence_base.DB):
@@ -414,7 +417,7 @@ class BasePersistenceTest(helper.TestHelper):
 
     def test_volume_get_all_by_host(self):
         # Only test when using our fake DB class.  We cannot use
-        # unittest2.skipUnless because persistence is configure in setUpClass,
+        # unittest.skipUnless because persistence is configure in setUpClass,
         # which is called after the decorator.
         if not isinstance(cinderlib.objects.Backend.persistence.db,
                           persistence_base.DB):
@@ -442,7 +445,7 @@ class BasePersistenceTest(helper.TestHelper):
 
     def test__volume_admin_metadata_get(self):
         # Only test when using our fake DB class.  We cannot use
-        # unittest2.skipUnless because persistence is configure in setUpClass,
+        # unittest.skipUnless because persistence is configure in setUpClass,
         # which is called after the decorator.
         if not isinstance(cinderlib.objects.Backend.persistence.db,
                           persistence_base.DB):
@@ -457,7 +460,7 @@ class BasePersistenceTest(helper.TestHelper):
 
     def test__volume_admin_metadata_update(self):
         # Only test when using our fake DB class.  We cannot use
-        # unittest2.skipUnless because persistence is configure in setUpClass,
+        # unittest.skipUnless because persistence is configure in setUpClass,
         # which is called after the decorator.
         if not isinstance(cinderlib.objects.Backend.persistence.db,
                           persistence_base.DB):
@@ -480,7 +483,7 @@ class BasePersistenceTest(helper.TestHelper):
 
     def test__volume_admin_metadata_update_do_nothing(self):
         # Only test when using our fake DB class.  We cannot use
-        # unittest2.skipUnless because persistence is configure in setUpClass,
+        # unittest.skipUnless because persistence is configure in setUpClass,
         # which is called after the decorator.
         if not isinstance(cinderlib.objects.Backend.persistence.db,
                           persistence_base.DB):
@@ -504,7 +507,7 @@ class BasePersistenceTest(helper.TestHelper):
 
     def test_volume_admin_metadata_delete(self):
         # Only test when using our fake DB class.  We cannot use
-        # unittest2.skipUnless because persistence is configure in setUpClass,
+        # unittest.skipUnless because persistence is configure in setUpClass,
         # which is called after the decorator.
         if not isinstance(cinderlib.objects.Backend.persistence.db,
                           persistence_base.DB):
@@ -520,3 +523,41 @@ class BasePersistenceTest(helper.TestHelper):
         result = self.persistence.db._volume_admin_metadata_get(self.context,
                                                                 vols[0].id)
         self.assertDictEqual({'k': 'v'}, result)
+
+    @mock.patch('cinderlib.objects.Volume.get_by_id')
+    @mock.patch('cinderlib.objects.Volume.snapshots',
+                new_callable=mock.PropertyMock)
+    @mock.patch('cinderlib.objects.Volume.connections',
+                new_callable=mock.PropertyMock)
+    def test_volume_refresh(self, get_conns_mock, get_snaps_mock, get_mock):
+        vol = self.create_n_volumes(1)[0]
+        vol_id = vol.id
+        # This is to simulate situation where the persistence does lazy loading
+        vol._snapshots = vol._connections = None
+        get_mock.return_value = cinderlib.Volume(vol)
+
+        vol.refresh()
+
+        get_mock.assert_called_once_with(vol_id)
+        get_conns_mock.assert_not_called()
+        get_snaps_mock.assert_not_called()
+        self.assertIsNone(vol.local_attach)
+
+    @mock.patch('cinderlib.objects.Volume.get_by_id')
+    @mock.patch('cinderlib.objects.Volume.snapshots',
+                new_callable=mock.PropertyMock)
+    @mock.patch('cinderlib.objects.Volume.connections',
+                new_callable=mock.PropertyMock)
+    def test_volume_refresh_with_conn_and_snaps(self, get_conns_mock,
+                                                get_snaps_mock, get_mock):
+        vol = self.create_n_volumes(1)[0]
+        vol_id = vol.id
+        vol.local_attach = mock.sentinel.local_attach
+        get_mock.return_value = cinderlib.Volume(vol)
+
+        vol.refresh()
+
+        get_mock.assert_called_once_with(vol_id)
+        get_conns_mock.assert_called_once_with()
+        get_snaps_mock.assert_called_once_with()
+        self.assertIs(mock.sentinel.local_attach, vol.local_attach)
