@@ -21,6 +21,7 @@ from unittest import mock
 from cinder import utils
 import ddt
 from oslo_config import cfg
+from oslo_privsep import priv_context
 
 import cinderlib
 from cinderlib import objects
@@ -103,6 +104,47 @@ class TestCinderlib(base.BaseTest):
         self.assertEqual(('default',), backend.pool_names)
         mock_workarounds.assert_called_once_with(mock_config.return_value)
 
+    @mock.patch('cinderlib.Backend._apply_backend_workarounds')
+    @mock.patch('oslo_utils.importutils.import_object')
+    @mock.patch('cinderlib.Backend._get_backend_config')
+    @mock.patch('cinderlib.Backend.global_setup')
+    def test_init_setup(self, mock_global_setup, mock_config, mock_import,
+                        mock_workarounds):
+        """Test initialization with the new 'setup' driver method."""
+        cfg.CONF.set_override('host', 'host')
+        driver_cfg = {'k': 'v', 'k2': 'v2', 'volume_backend_name': 'Test'}
+        cinderlib.Backend.global_initialization = False
+        driver = mock_import.return_value
+        driver.do_setup.side_effect = AttributeError
+        driver.capabilities = {'pools': [{'pool_name': 'default'}]}
+
+        backend = objects.Backend(**driver_cfg)
+
+        mock_global_setup.assert_called_once_with()
+        self.assertIn('Test', objects.Backend.backends)
+        self.assertEqual(backend, objects.Backend.backends['Test'])
+        mock_config.assert_called_once_with(driver_cfg)
+
+        conf = mock_config.return_value
+        mock_import.assert_called_once_with(conf.volume_driver,
+                                            configuration=conf,
+                                            db=self.persistence.db,
+                                            host='host@Test',
+                                            cluster_name=None,
+                                            active_backend_id=None)
+        self.assertEqual(backend.driver, driver)
+        driver.do_setup.assert_called_once_with(objects.CONTEXT)
+        driver.check_for_setup_error.assert_not_called()
+        driver.setup.assert_called_once_with(objects.CONTEXT)
+        driver.init_capabilities.assert_called_once_with()
+        driver.set_throttle.assert_called_once_with()
+        driver.set_initialized.assert_called_once_with()
+        self.assertEqual(driver_cfg, backend._driver_cfg)
+        self.assertIsNone(backend._volumes)
+        driver.get_volume_stats.assert_not_called()
+        self.assertEqual(('default',), backend.pool_names)
+        mock_workarounds.assert_called_once_with(mock_config.return_value)
+
     @mock.patch.object(objects.Backend, 'global_initialization', True)
     @mock.patch.object(objects.Backend, '_apply_backend_workarounds')
     @mock.patch('oslo_utils.importutils.import_object')
@@ -397,7 +439,7 @@ class TestCinderlib(base.BaseTest):
     def test_stats_single(self):
         stat_value = {'driver_version': 'v1', 'key': 'value'}
         expect = {'driver_version': 'v1', 'key': 'value',
-                  'pools': [{'key': 'value', 'pool_name': 'fake_backend'}]}
+                  'pools': [{'key': 'value', 'pool_name': self.backend_name}]}
         with mock.patch.object(self.backend.driver, 'get_volume_stats',
                                return_value=stat_value) as mock_stat:
             res = self.backend.stats(mock.sentinel.refresh)
@@ -506,6 +548,15 @@ class TestCinderlib(base.BaseTest):
         self.assertEqual(mock_conf.list_all_sections.return_value,
                          mock_conf.list_all_sections())
 
+    def _check_privsep_root_helper_opt(self, is_changed):
+        for opt in priv_context.OPTS:
+            if opt.name == 'helper_command':
+                break
+        helper_path = os.path.join(os.path.dirname(cinderlib.__file__),
+                                   'bin/venv-privsep-helper')
+        self.assertIs(is_changed,
+                      f'mysudo {helper_path}' == opt.default)
+
     @mock.patch.dict(os.environ, {}, clear=True)
     @mock.patch('os.path.exists')
     @mock.patch('configparser.ConfigParser')
@@ -527,6 +578,7 @@ class TestCinderlib(base.BaseTest):
             mock_ctxt_init.assert_not_called()
             self.assertIs(original_helper_func, utils.get_root_helper)
             self.assertIs(rootwrap_config, cfg.CONF.rootwrap_config)
+            self._check_privsep_root_helper_opt(is_changed=False)
         finally:
             cfg.CONF.rootwrap_config = original_rootwrap_config
 
@@ -582,6 +634,7 @@ class TestCinderlib(base.BaseTest):
 
             self.assertIs(original_helper_func, utils.get_root_helper)
             self.assertEqual(venv_wrap_cfg, cfg.CONF.rootwrap_config)
+            self._check_privsep_root_helper_opt(is_changed=True)
         finally:
             cfg.CONF.rootwrap_config = original_rootwrap_config
             utils.get_root_helper = original_helper_func
@@ -657,6 +710,7 @@ class TestCinderlib(base.BaseTest):
 
             self.assertIs(original_helper_func, utils.get_root_helper)
             self.assertEqual(venv_wrap_cfg, cfg.CONF.rootwrap_config)
+            self._check_privsep_root_helper_opt(is_changed=True)
         finally:
             cfg.CONF.rootwrap_config = original_rootwrap_config
             utils.get_root_helper = original_helper_func
